@@ -16,7 +16,7 @@ from __future__ import annotations as IMPORT_future_annotations
 # may be invalid (broken) if the generated code is mismatched with the installed
 # libraries.
 import rebootdev.versioning as IMPORT_reboot_versioning
-IMPORT_reboot_versioning.check_generated_code_compatible("0.38.2")
+IMPORT_reboot_versioning.check_generated_code_compatible("0.40.1")
 
 # ATTENTION: no types in this file should be imported with their unqualified
 #            name (e.g. `from typing import Any`). That would cause clashes
@@ -48,6 +48,7 @@ from datetime import timedelta as IMPORT_datetime_timedelta
 from datetime import timezone as IMPORT_datetime_timezone
 from google.protobuf import timestamp_pb2 as IMPORT_google_protobuf_timestamp_pb2
 from google.protobuf import wrappers_pb2 as IMPORT_google_protobuf_wrappers_pb2
+from google.protobuf.empty_pb2 import Empty as IMPORT_google_protobuf_empty_pb2_Empty
 import rebootdev.aio.tracing as IMPORT_reboot_aio_tracing
 from google.rpc import status_pb2 as IMPORT_google_rpc_status_pb2
 from tzlocal import get_localzone as IMPORT_tzlocal_get_localzone
@@ -73,9 +74,8 @@ from rebootdev.time import DateTimeWithTimeZone as IMPORT_reboot_time_DateTimeWi
 import rbt.v1alpha1 as IMPORT_rbt_v1alpha1
 import rbt.v1alpha1.nodejs_pb2 as IMPORT_rbt_v1alpha1_nodejs_pb2
 import google.protobuf.any_pb2 as IMPORT_google_protobuf_any_pb2
+import sys as IMPORT_sys
 
-# User defined or referenced imports.
-import google.protobuf.descriptor_pb2
 # Additionally re-export all messages and enums from the pb2 module.
 from google.protobuf.descriptor_pb2 import (
     DescriptorProto,
@@ -104,8 +104,16 @@ from google.protobuf.descriptor_pb2 import (
     UninterpretedOption,
 )
 
+# User defined or referenced imports.
+import google.protobuf.descriptor_pb2
+
 logger = IMPORT_log_log.get_logger(__name__)
 
+# We won't validate Pydantic state models while they are under construction.
+states_being_constructed: set[str] = set()
+
+
+
 ############################ Legacy gRPC Servicers ############################
 # This section is relevant (only) for servicers that implement a legacy gRPC
 # service in a Reboot context. It is irrelevant to clients.
