@@ -8,6 +8,20 @@ import pickle
 from dataclasses import dataclass
 from log.log import get_logger, set_log_level
 from mcp.server import fastmcp
+from mcp.server.auth.middleware.auth_context import (
+    AuthContextMiddleware,
+    get_access_token,
+)
+from mcp.server.auth.middleware.bearer_auth import (
+    BearerAuthBackend,
+    RequireAuthMiddleware,
+)
+from mcp.server.auth.provider import (
+    OAuthAuthorizationServerProvider,
+    ProviderTokenVerifier,
+    TokenVerifier,
+)
+from mcp.server.auth.settings import AuthSettings
 from mcp.server.elicitation import (
     AcceptedElicitation,
     CancelledElicitation,
@@ -25,8 +39,9 @@ from mcp.shared.message import ServerMessageMetadata
 from rbt.mcp.v1.session_rbt import Session
 from rbt.v1alpha1.errors_pb2 import StateNotConstructed
 from reboot.aio.applications import Application
+from rebootdev.aio.servicers import Servicer
 from reboot.aio.contexts import EffectValidation, WorkflowContext
-from reboot.aio.external import ExternalContext
+from reboot.aio.external import ExternalContext, InitializeContext
 from reboot.aio.types import StateRef
 from reboot.aio.workflows import at_least_once
 from reboot.mcp.event_store import (
@@ -34,6 +49,8 @@ from reboot.mcp.event_store import (
     replay,
     qualified_stream_id,
 )
+from reboot.protobuf import from_model
+from google.protobuf import struct_pb2
 from reboot.mcp.servicers.session import (
     SessionServicer,
     _servers,
@@ -41,18 +58,20 @@ from reboot.mcp.servicers.session import (
 )
 from reboot.mcp.servicers.stream import StreamServicer
 from reboot.std.collections.v1 import sorted_map
-from rebootdev.aio.headers import CONSENSUS_ID_HEADER, STATE_REF_HEADER
+from rebootdev.aio.headers import SERVER_ID_HEADER, STATE_REF_HEADER
 from rebootdev.aio.backoff import Backoff
 from rebootdev.memoize.v1.memoize_rbt import Memoize
 from rebootdev.settings import DOCS_BASE_URL
 from starlette.applications import Starlette
 from starlette.datastructures import MutableHeaders
+from starlette.middleware import Middleware
+from starlette.middleware.authentication import AuthenticationMiddleware
 from starlette.requests import Request
 from starlette.responses import Response, StreamingResponse
 from starlette.routing import Route
 from starlette.types import Receive, Scope, Send
 from types import MethodType
-from typing import Any, Callable, Literal, Protocol, TypeAlias, cast
+from typing import Any, Awaitable, Callable, Literal, Protocol, TypeAlias, cast
 from uuid import uuid4, uuid5
 from uuid7 import create as uuid7  # type: ignore[import-untyped]
 
@@ -314,18 +333,47 @@ class DurableMCP:
     _resources: list[Resource]
     _prompts: list[Prompt]
     _tools: list[Tool]
+    _auth: AuthSettings | None
+    _auth_server_provider: OAuthAuthorizationServerProvider[Any, Any, Any] | None
+    _token_verifier: TokenVerifier | None
 
     def __init__(
         self,
         *,
         path: str,
         log_level: LogLevel = "WARNING",
+        auth: AuthSettings | None = None,
+        auth_server_provider: (
+            OAuthAuthorizationServerProvider[Any, Any, Any] | None
+        ) = None,
+        token_verifier: TokenVerifier | None = None,
     ):
         self._log_level = log_level
         self._path = path
         self._resources = []
         self._prompts = []
         self._tools = []
+        self._auth = auth
+        self._auth_server_provider = auth_server_provider
+        self._token_verifier = token_verifier
+
+        # Validate auth configuration (same as `FastMCP`).
+        if self._auth is not None:
+            if auth_server_provider and token_verifier:
+                raise ValueError(
+                    "Cannot specify both auth_server_provider and "
+                    "token_verifier"
+                )
+            if not auth_server_provider and not token_verifier:
+                raise ValueError(
+                    "Must specify either auth_server_provider or "
+                    "token_verifier when auth is enabled"
+                )
+        elif auth_server_provider or token_verifier:
+            raise ValueError(
+                "Cannot specify auth_server_provider or token_verifier "
+                "without auth settings"
+            )
 
         set_log_level(logging.getLevelNamesMapping()[log_level])
 
@@ -562,12 +610,38 @@ class DurableMCP:
             )
         )
 
-    def application(self) -> Application:
+    def application(
+        self,
+        *,
+        servicers: list[type[Servicer]] | None = None,
+        initialize: Callable[[InitializeContext], Awaitable[None]]
+        | None = None,
+    ) -> Application:
         """
+        :param servicers: (optional) the types of Reboot-powered servicers that
+                          this Application will serve.
+        
+        :param initialize: (optional) will be called after the Application's
+                           servicers have started for the first time, so that
+                           it can perform initialization logic (e.g.,
+                           constructing some well-known durable data structures,
+                           loading some data, etc. It must do so in the context
+                           of the given InitializeContext.
+       
         Returns a Reboot `Application` for running the MCP tools,
         resources, prompts, etc that were defined.
         """
-        application = Application(servicers=self.servicers())
+
+        async def default_initialize(context: InitializeContext) -> None:
+            # Do any app internal initialization here.
+
+            if initialize is not None:
+                await initialize(context)
+
+        application = Application(
+            servicers=(servicers or []) + self.servicers(),
+            initialize=default_initialize,
+        )
 
         application.http.mount(
             self._path,
@@ -585,6 +659,9 @@ class DurableMCP:
             self._resources,
             self._prompts,
             self._tools,
+            self._auth,
+            self._auth_server_provider,
+            self._token_verifier,
         )
 
 
@@ -594,9 +671,21 @@ def _streamable_http_app(
     resources: list[Resource],
     prompts: list[Prompt],
     tools: list[Tool],
+    auth: AuthSettings | None,
+    auth_server_provider: OAuthAuthorizationServerProvider[Any, Any, Any] | None,
+    token_verifier: TokenVerifier | None,
     external_context_from_request: Callable[[Request], ExternalContext],
 ):
-    mcp = fastmcp.FastMCP(log_level=log_level)
+    # Create token verifier from provider if needed (same as `FastMCP`).
+    if auth_server_provider and not token_verifier:
+        token_verifier = ProviderTokenVerifier(auth_server_provider)
+
+    mcp = fastmcp.FastMCP(
+        log_level=log_level,
+        auth=auth,
+        auth_server_provider=auth_server_provider,
+        token_verifier=token_verifier,
+    )
 
     for resource in resources:
         mcp.resource(
@@ -626,16 +715,37 @@ def _streamable_http_app(
 
     _servers[path] = mcp._mcp_server
 
+    # Create the `endpoint`.
+    endpoint = StreamableHTTPASGIApp(
+        path,
+        external_context_from_request,
+    )
+
+    # Add authentication middleware if `token_verifier` is configured.
+    middleware = None
+    if token_verifier:
+        middleware = [
+            Middleware(
+                AuthenticationMiddleware,
+                backend=BearerAuthBackend(token_verifier),
+            ),
+            Middleware(AuthContextMiddleware),
+        ]
+        # Wrap `endpoint` with auth requirement middleware.
+        required_scopes = (auth.required_scopes if auth else None) or []
+        resource_metadata_url = None
+        endpoint = RequireAuthMiddleware(
+            endpoint, required_scopes, resource_metadata_url
+        )
+
     return Starlette(
         routes=[
             Route(
                 "/{path:path}",
-                endpoint=StreamableHTTPASGIApp(
-                    path,
-                    external_context_from_request,
-                ),
+                endpoint=endpoint,
             ),
         ],
+        middleware=middleware,
     )
 
 
@@ -946,6 +1056,12 @@ def _wrap_tool(fn: mcp.types.AnyFunction) -> mcp.types.AnyFunction:
                 return await fn(**dict(bound.arguments))
 
             return fn(**dict(bound.arguments))
+        except PermissionError as e:
+            # Log authorization failures at `INFO` level without traceback.
+            logger.info(
+                f"Authorization denied in tool {fn.__name__}: {e}"
+            )
+            raise
         except:
             import traceback
             traceback.print_exc()
@@ -1027,7 +1143,7 @@ class StreamableHTTPASGIApp:
 
             # Need to delete consensus ID header so that we'll
             # properly get routed.
-            del headers[CONSENSUS_ID_HEADER]
+            del headers[SERVER_ID_HEADER]
 
             response: StreamingResponse | Response
 
@@ -1210,12 +1326,28 @@ class StreamableHTTPASGIApp:
                         # grab what is necessary and send if along
                         # in a picklable way.
                         message.metadata.request_context = None  # type: ignore
+
+                        # Extract `AccessToken` from `contextvar` and serialize
+                        # to protobuf `Value` (`contextvars` don't survive
+                        # across subprocess boundaries).
+                        access_token = get_access_token()
+                        if access_token is not None:
+                            access_token_value = from_model(
+                                access_token,
+                                by_alias=True,
+                                mode="json",
+                                exclude_none=True,
+                            )
+                        else:
+                            access_token_value = None
+
                         # TODO: ideally we spawn `HandleMessage`
                         # _before_ a 202 Accepted is sent.
                         await session.spawn().HandleMessage(
                             context,
                             path=self._path,
                             message_bytes=pickle.dumps(message),
+                            access_token=access_token_value,
                         )
 
                         if not isinstance(message, Exception) and isinstance(
