@@ -10,6 +10,9 @@ from anyio.streams.memory import (
 from contextlib import contextmanager
 from dataclasses import dataclass
 from log.log import get_logger
+from mcp.server.auth.middleware.auth_context import auth_context_var
+from mcp.server.auth.middleware.bearer_auth import AuthenticatedUser
+from mcp.server.auth.provider import AccessToken
 from mcp.server.lowlevel.server import Server
 from mcp.shared.message import ServerMessageMetadata, SessionMessage
 from rbt.mcp.v1.session_rbt import (
@@ -30,7 +33,8 @@ from reboot.mcp.event_store import (
     qualified_stream_id,
     replace_whole_floats_with_ints,
 )
-from reboot.protobuf import as_dict, from_model
+from reboot.protobuf import as_dict, as_model, from_model
+from rebootdev.aio.backoff import Backoff
 
 logger = get_logger(__name__)
 
@@ -126,7 +130,7 @@ class SessionServicer(Session.Servicer):
 
             await self.ref().per_workflow(
                 "Store stream",
-            ).Write(context, store_stream)
+            ).write(context, store_stream)
 
             stream = Stream.ref(stream_id)
 
@@ -166,7 +170,7 @@ class SessionServicer(Session.Servicer):
 
                 await self.ref().per_workflow(
                     "Store client info on initialize",
-                ).Write(context, store_client_info)
+                ).write(context, store_client_info)
 
             with self._get_request_streams(
                 request_id,
@@ -178,6 +182,11 @@ class SessionServicer(Session.Servicer):
                     context,
                     path=request.path,
                     message_bytes=request.message_bytes,
+                    access_token=(
+                        request.access_token
+                        if request.HasField("access_token")
+                        else None
+                    ),
                 )
 
                 async def send_and_receive():
@@ -233,16 +242,33 @@ class SessionServicer(Session.Servicer):
                             related_request_id=related_request_id,
                         )
 
-                        async def is_visual_studio_code():
-                            response = await self.ref().per_workflow(
-                                "Check if client is Visual Studio Code",
-                            ).get(context)
-                            assert response.HasField("client_info")
-                            if response.HasField("client_info"):
-                                return response.client_info.name == "Visual Studio Code"
-                            return False
+                        async def check_is_vscode():
+                            backoff = Backoff(max_backoff_seconds=2)
+                            while True:
+                                response = await self.ref().always().get(
+                                    context
+                                )
+                                if not response.HasField("client_info"):
+                                    await backoff()
+                                    continue
+                                # Technically `name` is required but
+                                # at least the MCP SDK doesn't
+                                # validate it via Pydantic, but Visual
+                                # Studio Code always seems to include
+                                # its name, so if we don't have a name
+                                # it is not Visual Studio Code.
+                                if response.client_info.HasField("name"):
+                                    return response.client_info.name == "Visual Studio Code"
+                                return False
 
-                        if await is_visual_studio_code():
+                        is_vscode = await at_least_once(
+                            "Check if client is Visual Studio Code",
+                            context,
+                            check_is_vscode,
+                            type=bool,
+                        )
+
+                        if is_vscode:
                             # For Visual Studio Code, also store the
                             # _outgoing_ message, i.e., event, on the
                             # aggregated stream.
@@ -331,8 +357,16 @@ class SessionServicer(Session.Servicer):
                     read_stream_send, _ = read_stream
 
                     await read_stream_send.send(message)
+
+                # Since we're not longer expecting a response for this
+                # event ID we can remove it. Note that this also
+                # ensure that during effect validation we won't try
+                # and process the message more than once.
+                del self._write_request_ids[event_id]
             else:
-                logger.info(f"Ignoring client response as server must have rebooted")
+                logger.info(
+                    "Ignoring unknown client response (maybe the server rebooted?)"
+                )
 
             return HandleMessageResponse()
 
@@ -358,6 +392,13 @@ class SessionServicer(Session.Servicer):
             _, read_stream_receive = read_stream
             write_stream_send, _ = write_stream
 
+            # Restore `AccessToken` from request to `contextvar`
+            # (`contextvars` don't survive across subprocess boundaries).
+            access_token: AccessToken | None = None
+            # Check if `optional` field is set (we pass `None` when no auth).
+            if request.HasField("access_token"):
+                access_token = as_model(request.access_token, AccessToken)
+
             async def cancel_outstanding_requests():
                 """
                 Helper that sends cancellations after a reboot for
@@ -434,6 +475,13 @@ class SessionServicer(Session.Servicer):
             async def server_run():
                 assert _context.get() is None
                 _context.set(context)
+
+                # Set `access_token` in `auth_context_var` if present.
+                auth_token = None
+                if access_token is not None:
+                    authenticated_user = AuthenticatedUser(access_token)
+                    auth_token = auth_context_var.set(authenticated_user)
+
                 try:
                     global _servers
                     server = _servers[path]
@@ -454,6 +502,8 @@ class SessionServicer(Session.Servicer):
                     raise
                 finally:
                     _context.set(None)
+                    if auth_token is not None:
+                        auth_context_var.reset(auth_token)
 
             # Run this as an asyncio task because it blocks when
             # calling `write_stream_send.send()`.
